
LoadAddOn("Blizzard_ArtifactUI")

local classIcon = 645226

local function AddPerks()
	local forceHide, header
	
	if not ArtifactFrame:IsShown() then
		forceHide = true
		SocketInventoryItem(16)
	end
	
	for i, powerID in ipairs(C_ArtifactUI.GetPowers()) do
		--local spellID, cost, currentRank, maxRank, bonusRanks, x, y, prereqsMet, isStart, isGoldMedal, isFinal = C_ArtifactUI.GetPowerInfo(powerID)
		local spellID, _, rank, maxRank, bonus = C_ArtifactUI.GetPowerInfo(powerID)
		local isStart, isGold = select(9, C_ArtifactUI.GetPowerInfo(powerID))
		local rankString = tostring(rank.."/"..maxRank)

		if rank > 0 and not isStart then
			if not header then
				header = true
				GameTooltip:AddDivider()
			end
			
			if bonus > 0 then
				maxRank = maxRank - bonus
				rank = rank - bonus
				rankString = tostring("|cFF00FF00(+"..bonus..")|r "..rank.."/"..maxRank)
			end
			
			GameTooltip:AddDoubleLine(GetSpellInfo(spellID), rankString, 1,1,1,1,1,1)
		end
	end	
	
	if ArtifactFrame:IsShown() and forceHide then
		HideUIPanel(ArtifactFrame)
	end
end

local function Artifact_OnClick(self)
	if HasArtifactEquipped() then
		local frame = ArtifactFrame
		local activeID = C_ArtifactUI.GetArtifactInfo()
		local equippedID = C_ArtifactUI.GetEquippedArtifactInfo()
		
		if frame:IsShown() and activeID == equippedID then
			HideUIPanel(frame)
		else
			SocketInventoryItem(16)
		end
	end
end

local function Artifact_OnUpdate(self)
	if HasArtifactEquipped() then
		local name, icon, totalXP, pointsSpent = select(3, C_ArtifactUI.GetEquippedArtifactInfo())
		local numPointsAvailableToSpend, xp, xpForNextPoint = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP)
		local points, xp, xpMax = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP)
		local perc = math.ceil(xp*100/xpMax)
		
		self.Icon:SetTexture(icon)
		--self:SetText(totalXP.." "..POWER_TYPE_POWER)
		self:SetText(perc.."%")
	else
		self.Icon:SetTexture(classIcon)
		self:SetText(ITEM_QUALITY6_DESC)
	end
end

local function Artifact_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOP")
	
	if HasArtifactEquipped() then
		local title, r,g,b = select(2, C_ArtifactUI.GetEquippedArtifactArtInfo())
		local name, icon, totalXP, pointsSpent = select(3, C_ArtifactUI.GetEquippedArtifactInfo())
		local points, xp, xpMax = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP)

		GameTooltip:AddLine(title, r,g,b, false)
		--GameTooltip:SetPrevLineJustify("CENTER")
		GameTooltip:AddDivider()
		if points > 0 then
			GameTooltip:AddDoubleLine("Available Trait Points:", points, 1,1,1, 1,1,1)
		end
		GameTooltip:AddDoubleLine("Next Trait Point in:", xpMax-xp, 1,1,1, 1,1,1)
			
		--AddPerks()
	else
		GameTooltip:AddLine(SPELL_FAILED_NO_ARTIFACT_EQUIPPED, 1,1,1)
	end
	
	GameTooltip:Show()
end

do	-- Initialize
	local info = {}

	info.title = ITEM_QUALITY6_DESC
	info.icon = classIcon
	info.clickFunc = Artifact_OnClick
	info.updateFunc = Artifact_OnUpdate
	info.tooltipFunc = Artifact_OnEnter
	
	SyncUI_RegisterBrokerType("Artifact", info)
end
