 
local version = GetAddOnMetadata("SyncUI", "Version")

local maxLines, ChangeLogData = 5, {
	"Changes:",
		"• 7.1: API Updates",
}

local function ModifiedString(string)
	local newString = string
	local count = string:find(":")	
	
	-- header + subHeader
	if count then
		local prefix = string:sub(0, count)
		local suffix = string:sub(count+1)
		local subHeader = string:find("•")
		
		if subHeader then
			newString = tostring("|cFF66FF00"..prefix.."|r"..suffix)
		else
			newString = tostring("|cFFFF6600"..prefix.."|r"..suffix)
		end
	end

	-- highlights
	for pattern in string:gmatch("('.*')") do
		newString = newString:gsub(pattern, "|cFFFF8800"..pattern:gsub("'","").."|r")
	end
	
	return newString
end

local function GetChangeLogInfo(i)
	for line, info in pairs(ChangeLogData) do
		if line == i then
			return info
		end
	end
end


function SyncUI_ToggleChangeLog()
	local frame = SyncUI_ChangeLog
	local isPlaying = frame.SlideIn:IsPlaying() or frame.SlideOut:IsPlaying()

	if not isPlaying then
		PlaySound("igMainMenuOptionCheckBoxOn")
		
		if frame:IsShown() then
			frame.SlideOut:Play()
		elseif not SyncUI_OptionsMenu:IsShown() then
			frame.SlideIn:Play()
		end
	end
end

function SyncUI_ChangeLog_OnLoad(self)
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self.Version:SetText("|cFF7FFE00Sync|rUI - "..version)

	tinsert(UISpecialFrames,self:GetName())
end

function SyncUI_ChangeLog_OnCheckVersion(self)
	if not SyncUI_Data["Version"] or ( SyncUI_Data["Version"] and SyncUI_Data["Version"] ~= version) then
		SyncUI_Data["Version"] = version
		SyncUI_ToggleChangeLog()
	end
	
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

function SyncUI_ChangeLog_OnScrollUpdate(self)
	local offset = FauxScrollFrame_GetOffset(self)
	local parent = self:GetParent()
	
	SyncUI_ScrollFrame_Update(self, #ChangeLogData, maxLines, 30)
	
	for i = 1, maxLines do
		local button = parent["Button"..i]
		local idx = offset + i
		
		if idx <= #ChangeLogData then
			local string = ModifiedString(GetChangeLogInfo(idx))
			
			button.Text:SetText(string)
			button.tipText = string
			
			--[[
			if button:IsMouseOver() then
				button:GetScript("OnEnter")(button)
			end
			--]]
			
			button:Show()
		else
			button:Hide()
		end
	end
end
